@@ -12,15 +12,14 @@
 /*************************************************************************/
 
 #include <math.h>
+#include <string.h>
 #include <link-grammar/api.h>
 #include "disjuncts.h"
 #include "error.h"
 #include "preparation.h"
 #include "read-regex.h"
 #include "regex-morph.h"
-#ifdef USE_SAT_SOLVER
 #include "sat-solver/sat-encoder.h"
-#endif
 #include "corpus/corpus.h"
 #include "spellcheck-hun.h"
 
@@ -80,6 +79,7 @@ Parse_Options parse_options_create(void)
 	po->null_block = 1;
 	po->islands_ok = FALSE;
 	po->use_spell_guess = TRUE;
+	po->use_sat_solver = FALSE;
 	po->cost_model.compare_fn = &VDAL_compare_parse;
 	po->cost_model.type = VDAL;
 	po->short_length = 6;
@@ -152,6 +152,18 @@ int parse_options_get_verbosity(Parse_Options opts) {
 	return opts->verbosity;
 }
 
+void parse_options_set_use_sat_parser(Parse_Options opts, int dummy) {
+#ifdef USE_SAT_SOLVER
+	opts->use_sat_solver = dummy;
+#else
+	prt_error("Error: cannot enable the Boolean SAT parser; this "
+	          " library was built without SAT solver support.");
+#endif
+}
+int parse_options_get_use_sat_parser(Parse_Options opts) {
+	return opts->use_sat_solver;
+}
+
 void parse_options_set_linkage_limit(Parse_Options opts, int dummy) {
 	opts->linkage_limit = dummy;
 }
@@ -1010,7 +1022,7 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 	Sentence sent;
 
 	sent = (Sentence) xalloc(sizeof(struct Sentence_s));
-	bzero(sent, sizeof(struct Sentence_s));
+	memset(sent, 0, sizeof(struct Sentence_s));
 	sent->dict = dict;
 	sent->length = 0;
 	sent->num_linkages_found = 0;
@@ -1054,11 +1066,18 @@ static void set_is_conjunction(Sentence sent)
 	}
 }
 
-static int split_sentence(Sentence sent, Parse_Options opts)
+int sentence_split(Sentence sent, Parse_Options opts)
 {
 	int i;
 	Dictionary dict = sent->dict;
 
+	/* Cleanup stuff previously allocated. This is because some free 
+	 * routines depend on sent-length, which might change in different
+	 * parse-opts settings. 
+	 */
+	free_deletable(sent);
+
+	/* Tokenize */
 	if (!separate_sentence(sent, opts))
 	{
 		return -1;
@@ -1079,10 +1098,19 @@ static int split_sentence(Sentence sent, Parse_Options opts)
 	if (!(dict->unknown_word_defined && dict->use_unknown_word))
 	{
 		if (!sentence_in_dictionary(sent)) {
-			return -1;
+			return -2;
 		}
 	}
 
+	/* Look up each word in the dictionary, collect up all
+	 * plausible disjunct expressions for each word.
+	 */
+	if (!build_sentence_expressions(sent, opts))
+	{
+		sent->num_valid_linkages = 0;
+		return -3;
+	}
+
 	return 0;
 }
 
@@ -1168,28 +1196,56 @@ int sentence_nth_word_has_disjunction(Sentence sent, int i)
 	return (sent->parse_info->chosen_disjuncts[i] != NULL);
 }
 
+static void chart_parse(Sentence sent, Parse_Options opts)
+{
+	int nl;
+
+	/* Build lists of disjuncts */
+	prepare_to_parse(sent, opts);
+
+	init_fast_matcher(sent);
+	init_table(sent);
+
+	/* A parse set may have been already been built for this sentence,
+	 * if it was previously parsed.  If so we free it up before
+	 * building another.  */
+	free_parse_set(sent);
+	init_x_table(sent);
+
+	for (nl = opts->min_null_count; nl<=opts->max_null_count ; ++nl)
+	{
+		s64 total;
+		if (resources_exhausted(opts->resources)) break;
+		sent->null_count = nl;
+		total = do_parse(sent, sent->null_count, opts);
+
+		/* Give up if the parse count is overflowing */
+		if (PARSE_NUM_OVERFLOW < total) break;
+
+		sent->num_linkages_found = (int) total;
+		print_time(opts, "Counted parses");
+		post_process_linkages(sent, opts);
+		if (sent->num_valid_linkages > 0) break;
+	}
+
+	free_table(sent);
+	free_fast_matcher(sent);
+}
+
 int sentence_parse(Sentence sent, Parse_Options opts)
 {
 	int rc;
-	int nl;
-	s64 total;
 
 	verbosity = opts->verbosity;
 
-	/* Cleanup stuff previously allocated. This is because some free 
-	 * routines depend on sent-length, which might change in different
-	 * parse-opts settings. 
+	/* If the sentence has not yet been split, do so now.
+	 * This is for backwards compatibility, for existing programs
+	 * that do not explicitly call the splitter.
 	 */
-	free_deletable(sent);
-
-	rc = split_sentence(sent, opts);
-	if (rc) return -1;
-
-	/* Tokenize, look up each word in the dictionary, collect up all
-	 * plausible disjunct expressions for each word. */
-	if (!build_sentence_expressions(sent, opts)) {
-		sent->num_valid_linkages = 0;
-		return 0;
+	if (0 == sent->length)
+	{
+		rc = sentence_split(sent, opts);
+		if (rc) return -1;
 	}
 
 	/* Initialize/free any leftover garbage */
@@ -1207,40 +1263,14 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	/* Expressions were previously set up during the tokenize stage. */
 	expression_prune(sent);
 	print_time(opts, "Finished expression pruning");
-#ifdef USE_SAT_SOLVER
-	sat_parse(sent, opts);
-#else
-
-	/* Build lists of disjuncts */
-	prepare_to_parse(sent, opts);
-
-	init_fast_matcher(sent);
-	init_table(sent);
-
-	/* A parse set may have been already been built for this sentence,
-	 * if it was previously parsed.  If so we free it up before
-	 * building another.  */
-	free_parse_set(sent);
-	init_x_table(sent);
-
-	for (nl = opts->min_null_count; nl<=opts->max_null_count ; ++nl)
+	if (opts->use_sat_solver)
 	{
-		if (resources_exhausted(opts->resources)) break;
-		sent->null_count = nl;
-		total = do_parse(sent, sent->null_count, opts);
-
-		/* Give up if the parse count is overflowing */
-		if (PARSE_NUM_OVERFLOW < total) break;
-
-		sent->num_linkages_found = (int) total;
-		print_time(opts, "Counted parses");
-		post_process_linkages(sent, opts);
-		if (sent->num_valid_linkages > 0) break;
+		sat_parse(sent, opts);
+	}
+	else
+	{
+		chart_parse(sent, opts);
 	}
-
-	free_table(sent);
-	free_fast_matcher(sent);
-#endif /* USE_SAT_SOLVER */
 	print_time(opts, "Finished parse");
 
 	return sent->num_valid_linkages;
